"use strict";
/* tslint:disable no-console */
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = exports.LogLevel = void 0;
const chalk = require("chalk");
const json_schema_to_typescript_1 = require("json-schema-to-typescript");
const lodash_1 = require("lodash");
const path = require("path");
const files_1 = require("./files");
var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["NONE"] = 0] = "NONE";
    LogLevel[LogLevel["ERROR"] = 1] = "ERROR";
    LogLevel[LogLevel["INFO"] = 2] = "INFO";
})(LogLevel = exports.LogLevel || (exports.LogLevel = {}));
let loggingLevel = LogLevel.NONE;
function logInfo(message) {
    if (loggingLevel >= LogLevel.INFO) {
        console.log(chalk.cyan.bold(message), 'SpecParser');
    }
}
function logSuccess(message) {
    if (loggingLevel >= LogLevel.INFO) {
        console.log(chalk.green.bold(message), 'SpecParser');
    }
}
function logError(message) {
    if (loggingLevel >= LogLevel.ERROR) {
        console.log(chalk.red.bold(message), 'SpecParser');
    }
}
function extractRefsFromSchema(inputSchema) {
    const objectSchema = inputSchema;
    const refSchema = inputSchema;
    switch (objectSchema.type) {
        case 'object':
            return Object.values(preprocessProperties(objectSchema.properties));
        case 'array':
            return extractRefsFromSchema(objectSchema.items);
        default:
            // TODO: Handle allOf, anyOf
            return refSchema.$ref || undefined;
    }
}
function preprocessProperties(inputProperties) {
    if (inputProperties) {
        return Object.entries(inputProperties).reduce((acc, [name, value]) => (Object.assign(Object.assign({}, acc), { [name]: extractRefsFromSchema(value) })), {});
    }
    return {};
}
async function createImport(name, schema, filePath) {
    const references = extractRefsFromSchema(schema);
    if (!references || !references.length) {
        return;
    }
    const refsArray = typeof references === 'string' ? [references] : references.flat();
    const $refs = refsArray
        .filter((val, index) => val !== undefined && !val.includes(name) && refsArray.indexOf(val) === index)
        .map((val) => val.split('/')[3]);
    if ($refs.length) {
        await files_1.appendFile(filePath, `import { ${$refs.join(', ')} } from './'; \n\n`);
    }
}
async function createInterfaceFile(name, interfacesDirPath) {
    const bannerComment = '/* tslint:disable */\n/**\n* This file was automatically generated.\n* DO NOT MODIFY IT BY HAND.\n*/\n\n';
    const filePath = `${interfacesDirPath}/${name}.ts`;
    await files_1.writeFile(filePath, bannerComment);
    return filePath;
}
function removeEnum(content) {
    const enumStartPosition = content.search('export enum');
    const interfaceStartPosition = content.search('export interface');
    if (enumStartPosition > -1 && interfaceStartPosition > -1) {
        const enumEndPosition = content.slice(enumStartPosition, content.length).indexOf('}') + enumStartPosition;
        const enumParts = content.slice(enumStartPosition, enumEndPosition + 1);
        content = content.replace(enumParts, '').trim();
        return removeEnum(content);
    }
    return content;
}
async function createInterfaceContent(name, openApiSpec, filePath) {
    const components = openApiSpec.components;
    const schema = openApiSpec.components.schemas[name];
    await createImport(name, schema, filePath);
    const options = {
        bannerComment: '',
        declareExternallyReferenced: false,
        enableConstEnums: false,
        unknownAny: false,
        unreachableDefinitions: false,
        strictIndexSignatures: false,
        style: {
            bracketSpacing: true,
            printWidth: 120,
            semi: true,
            singleQuote: true,
            tabWidth: 4,
            useTabs: false,
        },
    };
    const schemaAndDefinitions = Object.assign(Object.assign({}, schema), { components });
    let content = await json_schema_to_typescript_1.compile(schemaAndDefinitions, name, options);
    // TODO: check future updates from json-schema-to-typescript, as next step should not be needed
    if (!options.declareExternallyReferenced) {
        content = removeEnum(content);
    }
    await files_1.appendFile(filePath, content);
    return content;
}
async function createInterface(name, openApiSpec, interfacesDirPath) {
    await files_1.appendFile(`${interfacesDirPath}/index.ts`, `export * from './${name}';\n`);
    const filePath = await createInterfaceFile(name, interfacesDirPath);
    const content = await createInterfaceContent(name, openApiSpec, filePath);
    logInfo(`${name} interface generated`);
    return { content, filePath, name };
}
async function parseSchema(name, openApiSpec, interfacesDirPath) {
    const components = openApiSpec === null || openApiSpec === void 0 ? void 0 : openApiSpec.components;
    if (!components || !components.schemas || !components.schemas.hasOwnProperty(name)) {
        return null;
    }
    const schema = components.schemas[name];
    return schema ? createInterface(name, openApiSpec, interfacesDirPath) : null;
}
function delayedParsing(schemaKey, openApiSpec, interfacesDirPath, ms) {
    return new Promise((resolve, reject) => setTimeout(() => parseSchema(schemaKey, openApiSpec, interfacesDirPath)
        .then((res) => resolve(res))
        .catch((e) => reject(e)), ms));
}
async function openApiToInterfaces(openApiSpec, interfacesDirPath) {
    const schemasNames = Object.keys(openApiSpec.components.schemas).sort();
    await schemasNames.reduce(async (prevPromise, schemaKey) => {
        try {
            await prevPromise;
            return delayedParsing(schemaKey, openApiSpec, interfacesDirPath, 0);
        }
        catch (error) {
            logError(error.message);
            return Promise.resolve({});
        }
    }, Promise.resolve({}));
}
async function removeExistingInterfaces(interfacesPath) {
    const files = await files_1.readDir(interfacesPath);
    return Promise.all(Object.values(files).map((file) => files_1.removeFile(path.join(interfacesPath, file))));
}
function appendTitles(openApiSpec) {
    Object.entries(openApiSpec.components.schemas).forEach(([name, schema]) => {
        const objectSchema = schema;
        if (objectSchema.type) {
            objectSchema.title = objectSchema.title || name;
            if (objectSchema.type === 'string' && objectSchema.enum && !objectSchema.tsEnumNames) {
                objectSchema.tsEnumNames = objectSchema.enum.map((e) => lodash_1.snakeCase(e).toUpperCase());
            }
        }
    });
}
async function generate(openApiFilePath = './openapi.json', interfacesDirPath = './interfaces', logLevel = LogLevel.NONE) {
    loggingLevel = logLevel;
    try {
        const swaggerSpec = JSON.parse(await files_1.readFile(openApiFilePath));
        appendTitles(swaggerSpec);
        await removeExistingInterfaces(interfacesDirPath);
        await openApiToInterfaces(swaggerSpec, interfacesDirPath);
        logSuccess('Typescript interfaces generated');
    }
    catch (error) {
        logError(error.message);
        throw error;
    }
}
exports.generate = generate;
//# sourceMappingURL=spec-parser.js.map